function [ROut, VOut, sOut] =...
    L2toL2Gain(f,h,x,w, gamma, gainCons, Opt)

% For the system xdot = f(x,w), z = h(x), and given gain gamma
% this function computes a lower bound on the value of R such that
% x(0) = 0 & \|w\|_{2,T} <= R    ==>  \|z\|_{2,T} <= gamma*R



%%%=======================

if nargin ==7  
    if ~isfield(Opt,'R2High')
        Opt.R2High = 1000;
    end
    if ~isfield(Opt,'R2Low')
        Opt.R2Low = 0;
    end
    if ~isfield(Opt,'BisTol')
        Opt.BisTol = 1e-3;
    end
    if ~isfield(Opt,'MaxIter')
        Opt.MaxIter = 20;
    end
    if ~isfield(Opt,'StopTol')
        Opt.StopTol = 1e-3;
    end
    if ~isfield(Opt,'display')
        Opt.display = 1;
    end
else
    Opt.R2High = 1000;
    Opt.R2Low = 0;
    Opt.BisTol = 1e-3;
    Opt.MaxIter = 20;
    Opt.StopTol = 1e-3;
    Opt.display = 1;
end




%%%=====================

sBasis = gainCons.sBasis;
VBasis = gainCons.VBasis;
OptBis.R2Low = Opt.R2Low;
OptBis.R2High = Opt.R2High;
OptBis.R2Tol = Opt.BisTol;

ROut = 0;
VOut = polynomial(0);
sOut = polynomial(0);

[gL,VL] = linearizedGain(f,h,x,w,gainCons.l1);

if gamma > gL
    V = VL;
    k = 1;
    R2Current = 0;
    R2inc = 10;

    while (R2inc > Opt.StopTol) & (k <= Opt.MaxIter)
        [R2, s] = BisectionGivenV(f, h, x, w, V, gamma, sBasis, OptBis);
        if k == 1 && Opt.display
            fprintf('--R for initial V = %d \n', sqrt(R2));
        end
        R2AfterFixedV(k) = R2;
        sAfterFixedV(k) = s;
        OptBis.R2Low = R2;
        if R2 > 0
            %[R2, V] = BisectionGivenMultip(f, h, x, w, s, gamma, VBasis, OptBis);
            [isfeasp,V] = L2toL2sFixed(f, h, x, w, s, R2, gamma, VBasis);
            if isfeasp
                if Opt.display
                    fprintf('--Iteration = %d; R = %d \n', k,sqrt(R2));
                end
                ROut = sqrt(R2);
                VOut = V;
                sOut = s;
            end
        end
        R2AfterFixedMultip(k) = R2;
        OptBis.R2Low = R2;
        VAfterFixedMultip(k) = V;
        R2inc = R2 - R2Current;
        R2Current = R2;
        k = k + 1;
    end

end











