function [ROut, VOut, s4Out, s10Out ] =...
    Reach(f,x,w, p, beta, reachCons, Opt)

% For the system xdot = f(x,w) and given shape function p (with p(0)=0) and its level beta
% compute the largest value of R such that 
% x(0) = 0 & \|w\|_{2,T} <= R    ==>  p(x(t)) \leq beta for all  0<=t<=T.



%%%=======================

if nargin ==7  
    if ~isfield(Opt,'R2High')
        Opt.R2High = 1000;
    end
    if ~isfield(Opt,'R2Low')
        Opt.R2Low = 0;
    end
    if ~isfield(Opt,'BisTol')
        Opt.BisTol = 1e-3;
    end
    if ~isfield(Opt,'MaxIter')
        Opt.MaxIter = 30;
    end
    if ~isfield(Opt,'StopTol')
        Opt.StopTol = 1e-3;
    end
    if ~isfield(Opt,'display')
        Opt.display = 1e-3;
    end
else
    Opt.R2High = 100;
    Opt.R2Low = 0;
    Opt.BisTol = 1e-3;
    Opt.MaxIter = 20;
    Opt.StopTol = 1e-3;
    Opt.display = 1e-3;
end




%%%=====================

s4Basis = reachCons.s4Basis;
s10Basis = reachCons.s10Basis;
VBasis = reachCons.VBasis;
l1 = reachCons.l1;

R2Low = Opt.R2Low;
R2High = Opt.R2High;
R2Tol = Opt.BisTol;

ROut = 0;
VOut = polynomial(0);
s4Out = polynomial(0);
s10Out = polynomial(0);

[isfeasp, VL] = linearizedReach(f,x,w,l1);


if isfeasp
    V = VL;
    k = 1;
    R2Current = 0;
    R2inc = 10;

    while (R2inc > Opt.StopTol) & (k <= Opt.MaxIter)
        [R2, s4, s10] =BisectionR2GivenV(f, p, x, w, V,...
                beta, s4Basis, s10Basis, R2Low, R2High, R2Tol);
        if k == 1 && Opt.display && R2 == 0
            fprintf('Initial V is not feasible');
        end
%         R2AfterFixedV(k) = R2;
%         sAfterFixedV(k) = s;
        OptBis.R2Low = R2;
        if R2 > 0
            [isfeasp,V] = check_givenMultip(s4,s10,R2,f,beta,VBasis, x, w, p, l1);
            if isfeasp
                if Opt.display
                    fprintf('--Iteration = %d; R = %d \n', k,sqrt(R2));
                end
                ROut = sqrt(R2);
                VOut = V;
                s4Out = s4;
                s10Out = s10;
            end
        end
%         R2AfterFixedMultip(k) = R2;
%         OptBis.R2Low = R2;
%         VAfterFixedMultip(k) = V;
        R2inc = R2 - R2Current;
        R2Current = R2;
        k = k + 1;
    end

end











