%
% This file simulates the open loop response for the F/A-18 Full Model 
% described in f18full.m file. Here, we will generate initial condition
% response to the state with zero intial condition to the inputs.
% 

%==========================================================================
% Simulate using ode45


% Unit Conversion : Degree <--> Radian
d2r = pi/180;
r2d = 1/d2r;


%-------------------------------------------
% Initialization

% State Initial Value
V       =  250;         % Airspeed , ft/s
beta    =  0*d2r;       % Sideslip Angle, rad
alpha   =  0*d2r;       % Angle-of-attack, rad

p       =  0*d2r;         % Roll rate, rad/s
q       =  0.2*d2r;       % Pitch rate, rad/s
r       =  0*d2r;         % Yaw rate, rad/s

phi     =  0*d2r;       % Roll Angle, rad
theta   =  0*d2r;       % Pitch Angle, rad
psi     =  0*d2r;       % Yaw Angle, rad

% Stack Initial Condition for State
x = [V;beta;alpha;p;q;r;phi;theta;psi];  

t = [0 10];            % time scale 
u = zeros(3,1);        % Input Initialization

%---------------------------------------------
% Simulate
[t,y] = ode45(@f18full,t,x,[],u); 


%==========================================================================
% Plot Results

figure
plot(t,y(:,4)*r2d,'-r','LineWidth',2)
hold on 
plot(t,y(:,5)*r2d,'--b','LineWidth',2)
hold on 
plot(t,y(:,6)*r2d,'--g','LineWidth',2)
grid on 
legend('Roll Rate (deg/s)','Pitch Rate (deg/s)','Yaw Rate (deg/s)')
xlabel('time, sec')
ylabel('Rate (deg/s)')
