%
% This file generates the closed loop linear plants for both baseline and 
% revised control law.  The open loop plant has been generated by running
% the Lin_F18.m file in the ' F18 Reduced Model\Linear Model' folder. 
% 





%==========================================================================
%
% Baseline Closed Loop Model



% The baseline control law uses lateral acceleration (Ay) as one of the 
% feedback channel which is not included in the C matrix of Pss; hence, 
% we will have to augment the Ay in the output equation (Pss.C) 

% qbar =  1/2*rho*V^2;  % Dynamic pressure

%--------------------------------------------------------------------------
% Sideforce Coefficient for Ay feedback

% C_Y        = ( 0.5781*alpha^2 +0.2834*alpha -0.8615)*beta...
%              + (-4.4857e-001*alpha +  3.0794e-001)*d_RUD ...
%              +  ( 4.2702e-001*alpha -1.0472e-001)*d_AIL;
%          
%          
% x = [beta;p1;r;phi;alpha;q];  % State Vector
% u = [d_AIL; d_RUD; d_STAB];   % Input Vector

%--------------------------------------------------------------------------
% Output Equation

% Ay = qbar*S*(C_Y)/m/g;
% y = [Ay; p1; r; alpha;q];
% J = jacobian(y,u);
% H = y-J*u;

%--------------------------------------------------------------------------
% Form Linearized Open Loop Plant for Baseline Control Law

Pss_Baseline = Pdata.Pss_Baseline;


%--------------------------------------------------------------------------
% Baseline Control Law Realization
Ac = [ -1 ];
Bc = -[0 0 -4.9 0 0]; 
Cc = [0;-1;0];
Dc = -[0 -0.3 0 0 0; 0.5 0 1.1 0 0;  0 0 0 0.8 8];
Css_Baseline = ss(Ac,Bc,Cc,Dc);
Css_Baseline.StateName = 'xc';   % Controller State 
%--------------------------------------------------------------------------
% Negative Feedback of Plin and C
CL_Baseline = feedback(Pss_Baseline,Css_Baseline);

disp('Eigenvalue of Baseline Control Law')
disp('==================================')
damp(CL_Baseline)




%==========================================================================
%
% Revised Closed Loop Model


% The revised control law uses Ay, beta, and betadot as feedback 
% channels which are not included in the C matrix of Pss;; hence, 
% we will have to augment the output equation (Pss.C) with these terms. 
% For betadot feedback, we will use the linearized kinematic equation of 
% betadot as feedback meaurement. 


%-------------------------------------------------------------------------- 
% Revised Control Law

% Y    =  C_Y*qbar*S ;
% Czx = (0.53594*alpha -3.8841*alpha^2 -1.6387*beta^2) + ...
%       ( 0.027584 - 0.77719 *alpha)*d_STAB;
% betad1 =   ( Y  - qbar*S*Czx*beta)/m/V + g*(phi)/V + p1*alpha - r ;


%--------------------------------------------------------------------------
% Output Equation

% Ay = qbar*S*(C_Y)/m/g;
% y = [Ay; p1; r; alpha;beta;q;betad1];
% J = jacobian(y,u);
% H = y-J*u;

%--------------------------------------------------------------------------
%  Form Linearized Open Loop Plant for Revised Control Law

Pss_Revised = Pdata.Pss_Revised;


%--------------------------------------------------------------------------
% Revised Control Law Realization 

kbetadot = 2;           % beta dot feedback
kbeta    = 0.5;         %beta feedback

Ac = [-1 ];
Bc = -[0 0 -4.9 0 0 0 0]; 
Cc =  [0 ;-1; 0];
Dc = -[0 -0.8 0 0 -kbeta 0 -kbetadot; 0.5 0 1.1 0 0 0 0;  0 0 0 0.8 0 2.5 0];
Css_Revised = ss(Ac,Bc,Cc,Dc);
Css_Revised.StateName = 'xc';   % Controller State 

%--------------------------------------------------------------------------
% Negative Feedback of Plin and C
CL_Revised = feedback(Pss_Revised,Css_Revised);
disp('Eigenvalue of Revised Control Law')
disp('==================================')
damp(CL_Revised)
