%==========================================================================
%   F18_LinearizedLyap
%   Compute an estimate of the region of attraction for the  
%   F/A-18 using the Lyapunov function obtained via linearization.
%==========================================================================

%==========================================================================
% Add NL Tool Path
%==========================================================================

 
%==========================================================================
% Generate Model 
% State Ordering 
% x     = [beta  ;      p1      ; r     ;phi        ; alpha     ; q;   xc ];
% f     = [betad;      pd       ; rd   ;phid         ; alphad   ;qd;   xcd];
%==========================================================================



Baseline = 1;       % Set to zero for revised model 


if Baseline 
   load F18BaselineCubic;
else
   load F18RevisedCubic; 
end



%==========================================================================
% Construct Lyap function from linearization 
Vlin=linstab(f,x);
Vdeg = Vlin.maxdeg;



%==========================================================================
% Create multiplier and function to force dV/dt<0
z2 =  monomials(x, 1);
L2 =  1e-6*(x'*x);
z1 =  monomials(x,0:1);


%==========================================================================
% Create Shape Function
Dmax = diag([5*d2r 20*d2r 5*d2r 45*d2r 25*d2r 25*d2r 25*d2r]);  
N = inv(Dmax^2);   %  Scale by inverse of max state values
N = N/max(N(:));   %  Normalize by the largest entry
p = x'*N*x;


%==========================================================================
% Set options for SOS problem
opts = [];
opts.gmin = 0; 
opts.gmax = 50;
opts.bistol = 1e-6; % For Baseline Model


%==========================================================================
% Gamma Step: Solve the following problem 
% {x:V(x) <= gamma} is contained in {x:grad(V)*f <0}
% max gamma subject to 
%                 -[grad(V)*f + (gamma - V)*s2] in SOS, s2 in SOS
%==========================================================================


[gbnds,s2]=pcontain(jacobian(Vlin,x)*f+L2,Vlin,z2,opts);
g = gbnds(1);
fprintf('Linearized V Analysis : gamma = %4.6f\n',g);


%==========================================================================
% Beta Step: Solve the following problem 
% {x: p(x)) <= beta} is contained in {x: V(x) <= gamma}
% max beta subject to 
%                 -[(V - gamma) + (beta - p)*s1] in SOS, s1 in SOS
%==========================================================================


[bbnds,s1]=pcontain(Vlin-g,p,z1,opts);
b = bbnds(1);
fprintf('Linearized V Analysis : beta= %4.6f\n',b);