%% F18 Analysis Folder Description
%
% This file explains the contents and objectives of each of the F18 
% Analysis folders that are provided here.  
%
% *NOTE:* Users who are not interested in model development and formulation
%         can skip folders 1 and 2.  They can use the plants given in 
%         folder 3 and proceed with the analysis in folders 4-6.
 
 
%% Folder 1: 1_F18FullModel
%
% *Description:* 
%
% This folder contains a detailed and full 6 degree-of-freedom 9 State 
% description of the F/A-18 aircraft, including the aerodynamic equations 
% and physical parameters of the aircraft. 
%
% *USAGE:*  
%
% Run *|runf18full.m|* to generate initial condition response of the
%           F/A-18 aircraft. 
%
% *Files:* 
%
% *NOTE:* This model has been reduced further to perform all the analyses.
% We have not used this model for any of the analyses /results presented 
% here.  This model is given only for completeness. None of the analyses have
% been performed on this full model. 
%
% *|f18full.m|*  This file describes a 6 DoF / 9 state F/A-18 nonlinear model. 
%                This also includes the aerodynamic data and physical parameters 
%                of the aircraft. 
%
% *|runf18full.m:|*  This file simulates initial condition response of the 
%                    F/A-18 model described in f18full.m. 
 
%% Folder 2: 2_F18ReducedModel 
%
% This folder formulates and develops the reduced order model for analysis. 
% The models generated from this folder have been included in Folder 3: 
% F18 Analysis Plants. This section can be skipped if model formulation
% is not of interest.  
%% Folder 2a: Nonlinear Model
%
% *Description:* 
%
% This folder contains multiple files. Each file will be described below.
% The subsequent analysis compares the stability properties of two 
% different control laws: (i) Baseline , and (ii) Revised control law.  
% This file generates cubic order reduced models for the F/A-18 open loop
% plant and for the closed-loop systems with the baseline and revised
% controllers.
%
% *USAGE:*  
%
% (a)  Run *|F18CubicNL_BaseLine.m|* to generate Cubic Degree Polynomial 
%      Closed Loop Baseline model. 
%
% (b)  Run *|F18CubicNL_Revised.m|* to generate Cubic Degree Polynomial 
%      Closed Loop Revised model.
%
% *Files:* 
%
% *NOTE* : *The following files require the Multipoly toolbox.*
%
% *|F18CubicNL_OLoop.m:|*  This file generates a polynomial open loop
%                        nonlinear F/A-18 model.  
%
% *|F18CubicNL_BaseLine.m:|*  This file generates the Baseline Closed Loop cubic
%                    order F/A-18 Model. 
%
% *|LS_xcldot_BaseLine.m:|* This file is being used by the F18Clp_BaseLine.m 
%                       file to perform the least square approximation. 
%
% *|F18CubicNL_Revised.m:|*  This file generates the Revised Closed Loop cubic
%                    order F/A-18 Model. 
%
% *|LS_xcldot_Revised.m:|*  This file is being used by the F18Clp_Revised.m 
%                       file to perform the least square approximation. 
%
% *|f18_data.m:|*  This file contains all the physical parameters of the F/A-18.
%                  The Velocity state which assumed to be constant 
%                  is also included there. 
%
% *|AeroApprox.m:|*  This file contains approximation of some of the 
%                   aerodynamic coefficients. 
 
 
%% Folder 2b: LinearModel
%
% *Description:* 
%
% This folder contains multiple files.  Each files will be described below.
% This folder generates the open loop linear model for the F/A-18 aircraft.
% 
% *USAGE:*  
%
% (a) Run  *|Lin_F18.m|* to generate open loop linear F/A-18 plant for
% flight condition described inside this file. 
%
% *Files:* 
% 
% *|Lin_F18.m:|*  This file will trim & generate the open loop linear model for 
%            F/A-18 aircraft. The trimming and linearization is done based            
%            on the  nonlinear plant 'F18Reduced_OLoop.m' described in the 
%            "2_F18ReducedModel /NonlinearModel" Folder. 
% 
% *|f18_plant.m:|* This file contains the nonlinear plant 'F18CubicNL_OLoop.m' 
%            described in the "2_F18ReducedModel /NonlinearModel" Folder. 
%            This file can be used to simulate open loop response. 
%
% *|f18_sfcn.m:|*  This is a S-function for performing trimming. This file
%               is being used in the Lin_F18.m file. 
%
% *|f18_trim.mdl:|* This is a Simulink file, also used for trimming. This file
%               is being used in the Lin_F18.m file. 
 
 
 
 
%% Folder 3: 3_F18AnalysisPlants
%
% *Description:* 
%
% This folder contains all the linear and nonlinear plants for analysis. 
% For detailed description on how these models have been generated refer to
% Folder 2 and its subfolders. 
%
% *Files:* 
%
% *|F18BaselineCubic.MAT:|*  This MAT file contains the cubic order F/A-18 
%                    baseline closed loop model.  This will be used for nonlinear
%                    analysis. 
%
% *|F18RevisedCubic.MAT:|*  This MAT file contains the cubic order F/A-18  
%                   revised closed loop model.  This will be used for nonlinear
%                   analysis. 
%
% *|LinF18OLoop---.MAT:|*  These MAT files (starting with 'LinF18OLoop...' )
%                   contain two open loop linear plants. 
%                   The difference in these two linear plants are in the
%                   output equation (y = Cx + Du), as each control law uses
%                   different feedback channels.  The plant named "Pss_Baseline" 
%                   will be used to construct the baseline model whereas                  
%                   plant "Pss_Revised" is used for building up the revised model. 
 
%% Folder 4: 4_LinearAnalysis
%
% *Description:* 
%
% This folder generates the closed loop linear (i) baseline, (ii) revised
% models.  Then, linear robustness analysis have been performed on these
% two plants. 
%
% *USAGE:*  
%
% (a) Run *|F18_Lin_Analysis.m:|* to perform linear analysis on both the
% linearized baseline and revised model.  This file generates the linear
% closed loop plants and perform all the robustness analysis. 
%
% *Files:*
%
% *|F18_Lin_Analysis.m:|*  Run this file to performs linear robustness 
%                        analysis on the two closed loop linear models
%                        specified above.
%
% *|Gen_LinF18Clps.m:|*  This file generates closed loop F/A-18 models given the open loop plants.
%                        The open loop plant(s) should be named in the
%                        following way : 
%                              Pdata.Pss_Baseline
%                              Pdata.Pss_Revised 
%                        
%             *NOTE:* Use "2_F18ReducedModel/LinearModel/Lin_f18.m"
%             for generating both (i) baseline (ii) revised open loop linear 
%             models with correct ordering of state, input and output. 
%
% *|Gen_LinF18ClpPlants.m:|*  This file generates both (i) baseline and (ii) revised
%                             closed loop linear models for all the linear plants 
%                             provided in the "3_F18AnalysisPlants" folder
%                             as *|LinF18OLoop**.MAT:|* files. 
 
 
 
%% Folder 5: 5_MonteCarlo
%
% *Description:* 
%
% This folder contains files that will run Monte Carlo simulation on the
% closed loop nonlinear plants provided in the "3_F18AnalysisPlants" Folder.
%
% *USAGE:* 
%
% Both the files can be executed to perform the tasks mentioned below. 
%
% *Files:*
%
% *|Search4UnstableTraj.m:|*  This file solves the following via Monte Carlo
%                           min  x0'*N*x0   ( N := shape factor)
%      subject to:  dx/dt = f(x) diverges from the initial condition x0. 
%
% *|Search4VdotPos.m:|*  This file attempts to solve via Monte Carlo search:
%                   min  x0'*N*x0
%                   subject to:  dV/dt = grad(V)(x)*f(x) >0 
 
%% Folder 6: 6_ROAAnalysis
%
% *Description:* 
%
% This folder contains files that will perform the region-of-attraction
% analysis on the cubic order polynomial models described in the 
% Folder 1-3.
%
% *USAGE:* 
%
% Both the files can be executed to perform the tasks mentioned below. 
%
% *Files:*
%
% *|ROA1_F18_LinearizedLyap.m:|*  This file computes an estimate of the 
%           region of attraction for the F/A-18 using the Lyapunov function 
%           obtained via linearization. 
%
% *|ROA2_F18_VSIteration.m:|* This file computes an estimate of the 
%                 region of attraction for F/A-18 using the V-S Iteration. 
 

