function [Beta_vec, Active, Mcell, Gamma_vec, Vvec, UpB_vec, NominalBetaVec, infoOut] =...
    BandBmanager(sys, XList, ini_cell, roaconstr, opt, infoin)

% cell is described by lower an upper bounds: [lower1 lower2 ... lowerm
% upper1 upper2 ... upperm]

d = opt.display.BB;

if d
    fprintf('***Start B&B refinement***\n');
end
ReturnedEmpty = 0;

infoOut = [];Beta_vec = []; Active = []; Mcell = [];Gamma_vec = []; UpB_vec = [];

% I cannot exactly remember what this if statement is for
% but I think if (ini_cell is passed nonempty) & (infoin is passed
% appropriately) then B&B iterations will start from that initial partition
% with the provided data. If ini_cell is passed empty, then infoin has to
% have proper initial data.
% In wrapper.m, only the case where there is no prior data and ini_cell is
% non-empty is considered.
if ~isempty(ini_cell)
    Active = ones(size(ini_cell,1),1);
    Mcell = ini_cell;
    for i1 = 1:size(ini_cell,1);
        [fnom{i1},fver{i1}] = getf(sys,ini_cell(i1,:));
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if opt.cellBeta.Extrafs > 0
            if opt.cellBeta.Extrafs >= size(fver{i1},2)-1
                opt.cellBeta.Extrafs = -1;
            else
                NoOfVert = size(fver{i1},2);
                vvv = ceil(NoOfVert*rand(opt.cellBeta.Extrafs,1));
                vvv = unique(vvv);
                fnom{i1} = [fnom{i1} fver{i1}(:,vvv)];
            end
        end
        if opt.cellBeta.Extrafs == -1
            fnom{i1} = fver{i1};
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end
    if d
        fprintf('-------Start cellBetaCenter for the initial partition --\n');
    end
    % cellBetaCenter can be called for multiple problems at the same time.
    [Beta, V, Gamma, UpB, NominalBeta, infoaux] = cellBetaCenter(fnom,fver,XList, roaconstr, opt);
    if d 
        fprintf('-------End cellBetaCenter for the initial partition --\n');
        fprintf('Current Beta  = %4.3f, \t Number of active cells = %i\n',min(Beta), length(Beta));   
    end
    Vvec = V;
    Gamma_vec = Gamma;
    Beta_vec = Beta;
    UpB_vec = UpB;
    NominalBetaVec = NominalBeta;
    if nargout == 8
        for i1 = 1:size(ini_cell,1)
            infoaux(i1).Active = Active;
            infoaux(i1).Beta_vec = Beta_vec;
            infoaux(i1).Gamma_vec = Gamma_vec;
            infoaux(i1).Vvec = Vvec;
            infoaux(i1).UpB_vec = UpB_vec;
            infoaux(i1).Mcell = Mcell;
            infoaux(i1).NominalBetaVec = NominalBetaVec;
        end
        infoOut = infoaux;
    end       
else
    m = size(infoin.Mcell,2)/2;
    Active = infoin.Active;
    Beta_vec = infoin.Beta_vec;
    Gamma_vec = infoin.Gamma_vec;
    Vvec = infoin.Vvec;
    Mcell = infoin.Mcell;   
    UpB_vec = infoin.UpB_vec;
    NominalBetaVec = infoin.NominalBetaVec;
end


ActiveIdx = find(Active);
[stop_lower, worstind] = min(Beta_vec(ActiveIdx));
worstind = ActiveIdx(worstind);
stop_upper = NominalBetaVec(worstind);

i1 = 1;
% Start iterations
while i1 <= opt.BB.max_iter 
    if d
        fprintf('-------Start B&B iteration number = %i --\n', i1);
    end
  % tic 
    clear fnom fver
    [Beta_vec, Gamma_vec, Vvec, UpB_vec, Active, Mcell, NominalBetaVec, fnom, fver, prior] =...
        getNewCells(Beta_vec, Gamma_vec, Mcell, Vvec, UpB_vec, Active, NominalBetaVec, opt, sys, i1);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i10 = 1:length(fnom)
        if opt.cellBeta.Extrafs > 0
            if opt.cellBeta.Extrafs >= size(fver{i10},2)-1
                opt.cellBeta.Extrafs = -1;
            else
                NoOfVert = size(fver{i10},2);
                vvv = ceil(NoOfVert*rand(opt.cellBeta.Extrafs,1));
                vvv = unique(vvv);
                fnom{i10} = [fnom{i10} fver{i10}(:,vvv)];
            end
        end
        if opt.cellBeta.Extrafs == -1
            fnom{i10} = fver{i10};
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     keyboard
    %%---------------------------------------------------------------------
    if opt.parallel.useCluster 
        persist = [0 0 1 1 1 0];
        for i2 = 1:length(fnom)
            csin(i2).fnom = fnom(i2);
            csin(i2).fver = fver(i2);
            csin(i2).XList = XList;
            csin(i2).roaconstr = roaconstr;
            csin(i2).opt = opt;
            csin(i2).prior = prior(i2);
        end
        
        save TemporaryCsIn csin         
        csout = cluster_eval(opt.parallel.machinesToUse,'cluster_cellBetaCenter', csin,'smooth',persist);
        for i2 = 1:length(fnom)
            if isempty(csout(i2).Beta)
                ReturnedEmpty = 1;
                Beta(i2,1) = csin(i2).prior.Beta;
            else
                Beta(i2,1) = csout(i2).Beta;
            end
            if isempty(csout(i2).Gamma)
                Gamma(i2,1) = csin(i2).prior.Gamma;
            else
                Gamma(i2,1) = csout(i2).Gamma;
            end
            if isempty(csout(i2).V)
                V(i2,1) = csin(i2).prior.V;
            else
                V(i2,1) = csout(i2).V;
            end
            if isempty(csout(i2).UpB)
                UpB(i2,1) = inf;
            else
                UpB(i2,1) = csout(i2).UpB;
            end
            if isempty(csout(i2).NominalBeta)
                NominalBeta(i2,1) = inf;
            else
                NominalBeta(i2,1) = csout(i2).NominalBeta;
            end    
        end
        

        delete('/home/cluster/lock/*');
        pnet closeall

    else
            [Beta,V,Gamma,UpB,NominalBeta,infoaux] = cellBetaCenter(fnom,fver,XList, roaconstr, opt, prior);
    end
    
    
    %%---------------------------------------------------------------------
    % If, for some reason, Beta returned from cellBetaCenter is less than 
    % what we have already verified, 
    % don't take that results and for that problem keep the previous
    % results. 
    mBeta = length(Beta);
    for i2 = 1:mBeta
        if Beta_vec(end-mBeta+i2,1) <= Beta(i2)
            Beta_vec(end-mBeta+i2,1) = Beta(i2);
            Gamma_vec(end-mBeta+i2,1) = Gamma(i2);
            UpB_vec(end-mBeta+i2,1) = UpB(i2);
            NominalBetaVec(end-mBeta+i2,1) = NominalBeta(i2);
            Vvec(end-mBeta+i2,1) = V(i2);
        end
    end
    
    
    
    for i3 = 1:mBeta
        infoaux(i3).Active = Active;
        infoaux(i3).Beta_vec = Beta_vec;
        infoaux(i3).Gamma_vec = Gamma_vec;
        infoaux(i3).Vvec = Vvec;
        infoaux(i3).UpB_vec = UpB_vec;
        infoaux(i3).Mcell = Mcell;
        infoaux(i3).NominalBetaVec = NominalBetaVec;
    end
    if nargout == 8 infoOut(end+1:end+mBeta) = infoaux; end

    
    if ~isempty(opt.SaveResults.filename)
        filename = opt.SaveResults.filename;
        save(filename,'Beta_vec','Active', 'Mcell', 'Gamma_vec', 'Vvec', 'UpB_vec', 'NominalBetaVec');%,'infoOut');
    end 
    
    ActiveIdx = find(Active);
    [stop_lower, worstind] = min(Beta_vec(ActiveIdx));
    worstind = ActiveIdx(worstind);
    stop_upper = NominalBetaVec(worstind);
    if d
        fprintf('-------End B&B iteration number = %i --\n', i1);
        fprintf('Current Beta  = %4.3f, \t Number of active cells = %i\n',min(Beta_vec(Active==1)), length(find(Active==1)));
    end
    i1 = i1 + 1;
end

if d
    fprintf('***End of B&B refinement***\n');
end


