                                                                     
                                                                     
                                                                     
                                             
function out  = PVget(a,property)
% function Out = PVget(A,Property)
%
% DESCRIPTION
%   (INTERNAL FUNCTION)
%   Private get for polynomial objects.
%
% INPUTS
%   A: a polynomial
%   Property: polynomial property
%
% OUTPUTS
%   Out: Value of the property
%
% SYNTAX
%   Out = PVget(A,Property);

% 6/7/2002: PJS  Initial Coding
% 10/31/2002 PJS Added properties for scalar polynomials
% 1/31/2003: SP  Added 'matdim' as a getable/setable property
% 3/30/2003: PJS Removed 'matdim' as a getable/setable property
%                (use size and reshape instead)
% 2/27/2008: PJS  Properly order varname output for vector of pvars

switch property

    case 'PropNames'
        % These are getable and setable PUBLIC properties
        out.GPropNames = {'coefficient' ; 'degmat';  'varname'; ...
            'nterms'; 'nvars'; 'maxdeg'; 'mindeg'};

        out.SPropNames = {'coefficient' ; 'degmat'; 'varname'};

    case 'coefficient'

        [nra,nca] =size(a);
        nta = size(a.degmat,1);
        out = reshape(a.coefficient,nta,nra*nca);

    case 'degmat'

        out = a.degmat;

    case 'varname'

        if ispvar(a)
            nvar = length(a.varname);
            [nra,nca] =size(a);
            nta = size(a.degmat,1);
            if ( nra*nca== nvar )
                % For a matrix or vector of unique pvars, output the
                % variables in the order that they appear in a.  In 
                % general this will be different from the order in 
                % a.varname because the combine function causes a.varname 
                % to be in alphabetical order.
                out = char(a);
%                 acoef = reshape(a.coefficient,nta,nra*nca);
%                 adeg = a.degmat;
%                 avars = cell(nvar,1);
%                 for i2=1:nvar
%                     idx1 = find( acoef(i2,:) );
%                     idx2 = find( adeg(idx1,:) );
%                     avars(i2) = a.varname(idx2);
%                 end
%                 out = avars;
            else
                out = a.varname;
            end
        else
            out = a.varname;
        end
    case 'nterms'

        out = size(a.degmat,1);

    case 'nvars'

        out = size(a.degmat,2);

    case 'maxdeg'

        if all( size(a.degmat)==[0 0]) %isempty(a.degmat)
            out = [];
        else
            out = full(max(sum(a.degmat,2)));
        end

    case 'mindeg'

        if all( size(a.degmat)==[0 0]) %isempty(a.degmat)
            out = [];
        else
            out = full(min(sum(a.degmat,2)));
        end

    otherwise

        error(['Property ' property ' is Invalid']);

end



