function [BetaBnds, V, info] = RoaEst(fList, XList, roaconstr, opt, fverList)
% function [BetaBnds, V, info] = roa_est(fList, XList, roaconstr,  opt)
%
% DESCRIPTION 
%   Estimates the region of attraction for an autonomous system possibly
%   with uncertainties which enter affinely.
%   
% INPUTS 
%   fList: 
%   XList: 
%   roaconstr:
%   opt: 
%
% OUTPUTS 
%   BetaBnds:  [UB, LB]
%   V:
%   info: 
%
% SYNTAX 
%   [BetaBnds, V, info] = roa_est(fList, XList, roaconstr, opt)
%
% REFERENCE
%   Topcu, Packard, and Seiler,  "Local Stability Analysis Using 
%   Simulations and Sum-of-Squares Programming," to appear in Automatica.
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%

%ttotal = tic;
tic

info = struct('inputs',[],'SimLFG',[],'iteration',[],'BetaBnds',[],'V',[],'CPUTimeIter',[]);

% display('check linearization in roa_est_New1');

xeq = zeros(length(XList),1);
if size(fList,2) > 1
    [Anom,Aver,stabnom,stabver,iscellstab] = getLinearization([],fverList,XList,xeq);
else
    [Anom,Aver,stabnom,stabver,iscellstab] = getLinearization(fList,[],XList,xeq);
end



if ~iscellstab
    BetaBnds = [0 0];
    V = polynomial(0);
    info.BetaBnds = BetaBnds;
    info.V = V;
    info.iteration.getgamma.Gamma = 0;
    if opt.display.roaest
        fprintf('****"Nominal" system does not satisfy conditions on the linearization****');
    end
    return
end

% display('iscellstab in roa_est_New1 IS CHECKED');

% Set options
d = opt.display.roaest;
MaxIters = opt.coordoptim.MaxIters;
IterStopTol = opt.coordoptim.IterStopTol;

% Store inputs into an information structure for output
info.inputs.fList = fList;
info.inputs.XList = XList;
info.inputs.roaconstr = roaconstr;
info.inputs.opt = opt;

% display('RIGHT BEFORE SimLFG in roaestNEw')
% SimLFG: Run simulations, form LP, and generate candidate Lyapunov function
[BetaUB,V,info.SimLFG] = SimLFG(fList, XList, roaconstr, opt);


if BetaUB > 0
    [V, Gammabest, Betabest, info] = preIter(V, fList, XList, roaconstr, opt, info);
    opt.getbeta.maxbeta = min([opt.getbeta.maxbeta, BetaUB]);
    info = RunIterations(V, Gammabest, Betabest, fList, XList, roaconstr, opt, info);
    BetaBnds = info.BetaBnds;
    V = info.V;
else
    BetaBnds = [0 0];
    V = [];
end

    
