function [c,info,ep] = findVfeas_Yalmip(fList,A,b,constr,opt)

if 1
    % XXX PJS 5/21/2009 Updated to sosopt
    x = fList.var;
    n = length(x);
    Nsys = size(fList,2);
    
    % Get options
    Z = constr.zV;
    mu = constr.mu;
    L1 = constr.L1;
    L2 = constr.L2;
    L2LP = constr.L2LP;

    % Create decision variable
    [V,cV] = polydecvar('cV',Z,'vec');
    if ~opt.feasonly 
        pvar ep; 
    end
    ineq{1} = V-L1;
    Vquad = cleanpoly(Z,[],2)'*cV;
    for i0 = 1:Nsys
        flin = cleanpoly( fList(:,i0), [], 1);
        J = double(jacobian(flin,x));
        if any(J(:)~=0) && (Vquad~=0)
            % Compute -d(Vquad(x))/dx*flin - mu*Vquad(x)
            if opt.feasonly
                Vd = -(jacobian(Vquad,x)*flin) - mu*Vquad - L2;
            else
                %%% 4 is here for experimental purposes
                Vd = -(jacobian(Vquad,x)*flin) - mu*Vquad - 4*ep*L2LP; 
            end
            ineq{i0+1} = Vd;
        end
    end
    
    % Tack on LP constraints: -1000<cV(:)<1000 and -1000<ep<1000
    ncA = size(A,2);
    A = [A; eye(ncA); -eye(ncA)];
    b = [b; 100*ones(2*ncA,1)]; 
    
    % Solve with sosopt
    lpconstr.A = A;
    lpconstr.b = b;
    if opt.feasonly
        lpconstr.d = cV;
        [info,dopt,sossol]=sosopt(ineq,x,[],lpconstr,opt);        
    else    
        ineq{Nsys+2} = ep*L2LP-L2;
        lpconstr.d = [cV;ep];
        [info,dopt,sossol]=sosopt(ineq,x,-ep,lpconstr,opt);
    end
    isfeasp = info.feas;
    
    
    c = []; epout = [];
    if isfeasp 
        c = double( subs(cV,dopt(:,1),dopt(:,2)) );
        if ~opt.feasonly 
            ep = double( subs(ep,dopt(:,1),dopt(:,2)) );            
        end
    end
    ep = epout;
    info.dopt = dopt;
    info.sossol = sossol;   
    
else
    % Original Code
    optaux = sdpsettings;
    optaux.verbose = 0;

    x = fList.var;
    n = length(x);
    xs = sdpvar(n,1);

    Z = constr.zV;
    ind2 = find(cleanpoly(Z,[],2)~=0);
    Zs = sdpmonomials(xs,Z.mindeg:Z.maxdeg);
    Zs2 = Zs;
    Zs2(setdiff(1:length(Z),ind2)) = 0;

    mu = constr.mu;
    L1 = constr.L1;
    L1v = full(poly2basis(L1,Z));
    L2 = constr.L2;
    L2v = full(poly2basis(L2,Z));
    L2LP = constr.L2LP;
    L2LPv = full(poly2basis(L2LP,Z));
    L1s = L1v'*Zs;
    L2s = L2v'*Zs;
    L2LPs = L2LPv'*Zs;

    cV = sdpvar(length(Zs),1);

    % XXX PJS 5/20/2009 Updated to new mpvar syntax
    %cVpvar = mpvar( length(Z), 1, 'c');
    cVpvar = mpvar( 'c',length(Z), 1);

    F = set(-1000<cV(:)<1000);
    if ~opt.feasonly sdpvar ep; F = F + set(-1000<ep<1000); end
    V = cV'*Zs;
    F = F + set(sos(V-L1s));
    Vquad = Zs2'*cV;
    Vquadpvar = Z(ind2)'*cVpvar(ind2);

    Nsys = size(fList,2);

    for i0 = 1:Nsys
        J = double(cleanpoly(jacobian(fList(:,i0),x),[],0));
        flin = J*xs;
        if any(J(:)~=0) && (Vquadpvar~=0)
            % Compute -d(Vquad(x))/dx*flin - mu*Vquad(x)
            if opt.feasonly
                Vd = -(jacobian(Vquad,xs)*flin) - mu*Vquad - L2s;
            else
                %%% 4 is here for experimental purposes
                Vd = -(jacobian(Vquad,xs)*flin) - mu*Vquad - 4*ep*L2LPs;
            end
            F = F + set(sos(Vd));
        end
    end
    if ~opt.feasonly
        F = F + set(sos(ep*L2LPs-L2s));
    end

    if opt.feasonly
        alph = cV;
    else
        alph = [cV;ep];
    end


    F = F + set(A*alph-b<=0);



    if opt.feasonly
        [sol,m,B,residual] = solvesos(F,[],optaux);
    else
        [sol,m,B,residual] = solvesos(F,-ep,optaux);
    end

    isfeasp = 0;

    if sol.problem == 0 
        i1 = 1;flag = 1;
        while flag == 1 & i1 <= length(B) 
            if min(eig(B{i1})) < 1e-10
                flag = 0;
            end
            i1 = i1 + 1;
        end
        if flag == 1
            isfeasp = 1;
        end
    end

    c = []; epout = [];

    if isfeasp 
        c = double(cV);
        if ~opt.feasonly ep = double(ep); end
    end




    ep = epout;

    info.solinfo = sol;
    info.sossol.m = m;
    info.sossol.B = B;
    info.sossol.residual = residual;
end







