function info = getbeta(V, Gamma, XList, roaconstr,opt)
% function info = getbeta(V, Gamma, XList, p, opt)
%
% DESCRIPTION 
%
%   
% INPUTS 
%   V: 
%   Gamma: 
%   XList: 
%   p: 
%   opt: 
%
% OUTPUTS 
%   info: structure with fields Beta and S1
%
% SYNTAX 
%   info = getbeta(V, Gamma, XList, p, opt)
%
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%

% No error checking


% Set Options
d = opt.display;
BetaHigh = opt.maxbeta;
BetaLow = opt.minbeta;
BTOL = opt.bistol;
usesosp = opt.usesosp;
sedopts = opt.sedopts;

z1 = roaconstr.z1;
p = roaconstr.p;

% Initalize Values for Bisection
BetaOut = 0; 
S1Out = polynomial([]);  


if p.maxdeg == 2 & V.maxdeg == 2
    if 1
        % XXX PJS 5/21/2009 Updated to sossopt
        [info,dopt,sossol]=sosopt(p);
        P = sossol{1,3};
        [info,dopt,sossol]=sosopt(V);
        Q = sossol{1,3};        
    else
        % Original Code
        P = double(findsos(p));
        Q = double(findsos(V));
    end
    S1Out = polynomial(max(eigs(Q,P)));
    BetaOut = Gamma/S1Out;
else
    
    



    if BetaLow>0
        % Verify BetaLow is a true lower bound on the first pass
        Beta = BetaLow;
        BetaLow = 0;
    else
        Beta = BetaHigh/2;
    end

    % Perform Bisection
    while BetaHigh-BetaLow>BTOL
        % Display Bisection status
        if d
            fprintf('BetaLow  = %4.3f \t',BetaLow);
            fprintf('BetaTry = %4.3f \t',Beta);
            fprintf('BetaHigh = %4.3f \n',BetaHigh);
        end

        % Run Feasibility Problem
        [isfeasp, S1] = LOCALVcontainP(Beta, V , Gamma, XList, p, ...
            z1,usesosp,sedopts);
        if ~isfeasp
            BetaHigh = Beta;
        else
            BetaLow = Beta;
            BetaOut = Beta;
            S1Out = S1;
        end

        % Set Beta level for next feasibility problem
        Beta = 0.5*(BetaLow+BetaHigh);
    end

    % Set output values of Beta and S1
    if isempty(BetaOut) BetaOut = 0; end
end
info.Beta = BetaOut;
info.S1 = S1Out;

%------------------------------------------------------------------
% Local Function: 
%   Given fixed Beta, Gamma, and V, find S1 which satisifies:
%        -[ (Beta-p)*S1 + (V-Gamma) ] in SOS
%   This is Equation 7 in the Automatica Paper.
%------------------------------------------------------------------
function [isfeasp, S1] = LOCALVcontainP(Beta, V , Gamma, XList, p,...
    z1,usesosp,sedopts);

if 1 %usesosp
    % XXX PJS 5/21/2009 Updated old sosp code to new sosopt 
    % (Hard-coded to run sosopt rather than rely on usesosp option)
    S1 = polydecvar('S1',z1,'mat');
    ineq{1} = S1;
    ineq{2} =  -( (Beta -p)*S1 + (V -Gamma) );
    [info,dopt]=sosopt(ineq,XList,[],[],sedopts);
    isfeasp = info.feas;
    if isfeasp
        S1 = subs(S1,dopt(:,1),dopt(:,2));
    else
        S1 = [];
    end
    
else 
    prog = sosprogram(XList);
    
    [prog,S1] = sossosvar(prog,z1);
    
    ineq1 = -( (Beta -p)*S1 + (V -Gamma) );
    prog = sosineq(prog,ineq1);
    [prog,info] = sossolve(prog,[],[],'sedumi');
    
    isfeasp = double(check_feasp(info,'sedumi')==1);
    S1 = sosgetsol(prog, S1);
    
end
