%% Demonstration of the wrapper.m

%% No uncertainty - ROA estimation
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
f = [x1dot; x2dot];
%%
% Get the default values of options for the vector field
% and the state vector.
% If zV (monomials vector for V), p (shape factor), or the 
% options for the multipliers (and type of iteration need to 
% be specified, pass them to GetRoaOpts.
% Bis.flag = 1;
[roaconstr,opt,sys] = GetRoaOpts(f, x);
opt.sim.NumConvTraj = 40;
%%
% call the wrapper which in term calls roa_est_New1
opt.display.roaest = 0;
outputs = wrapper(sys,[],roaconstr,opt);
%%
% Beta upper and lower bounds:
outputs.RoaEstInfo.BetaBnds

%% Uncertain vector field evaluated at the center of the cell (No non-affine uncertainty dependence).
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
%%
% Nominal system
f = [x1dot; x2dot];
%%
% Introduce the uncertain parameter 
pvar d1
%%
% Specify its value
ini_cell = 0;
%%
% Form the uncertain vector field
f = f + d1*[-x2;0];
%%
% Get the options and system in the required form
[roaconstr,opt,sys] = GetRoaOpts(f, x);
opt.sim.NumConvTraj = 40;
%%
% call the wrapper which in term calls roa_est_New1 after trivial
% manipulation (to get the vector field for the center system).
outputs = wrapper(sys,ini_cell,roaconstr,opt);
%%
% Beta upper and lower bounds:
outputs.RoaEstInfo.BetaBnds


%% Robust ROA estimation for uncertain vector field
% Compute V for the center and evaluate at the vertices
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
%%
% Nominal system
f = [x1dot; x2dot];
%%
% Introduce the uncertain parameter 
pvar d1
%%
% Specify its value
% ini_cell = [d1min d1max d2min d2max ... dmmin dmmax]
% this is the only difference from the previous part
ini_cell = [-.2 .2];
%%
% Form the uncertain vector field
f = f + d1*[-x2;0];
%%
% Get the options and system in the required form
[roaconstr,opt,sys] = GetRoaOpts(f, x);
opt.sim.NumConvTraj = 40;
%%
% call the wrapper which in term calls cellBetaCenter after trivial
% manipulation.
outputs = wrapper(sys,ini_cell,roaconstr,opt);
%%
% Beta upper and lower bounds:
[outputs.CellBetaInfo.infoOut.nominalinfo.BetaBnds(1) outputs.CellBetaInfo.infoOut.Beta]


%% Run B&B for the system in the previous case

%% 
% Modify opt to run B&B and to change the display options
opt.BB.runBB = 1;
opt.BB.max_iter = 2;
opt.display.BB = 1;
%% 
% call the wrapper which calls cellBetaCenter after trivial
% manipulation.
outputs = wrapper(sys,ini_cell,roaconstr,opt);

%%
% Upper and lower bound: 
[min(outputs.BBInfo.UpB_vec) min(outputs.BBInfo.Beta_vec(outputs.BBInfo.Active==1))]


%%
% When there is non-affine appearances of the uncertain parameters, there
% is not much change. 


%% Robust ROA estimation for uncertain vector field with non-affine uncertainty dependence
% Compute V for the center and evaluate at the vertices
pvar x1 x2;
x = [x1;x2];
x1dot = -x2;
x2dot = x1+(x1^2-1)*x2;
%%
% Nominal system
f = [x1dot; x2dot];
%%
% Introduce the uncertain parameters
pvar d1 d2
%%
% Specify its value
% ini_cell = [d1min d1max d2min d2max ... dmmin dmmax]
% this is the only difference from the previous part
ini_cell = [-.2 .2 -.1 .2];
%%
% Form the uncertain vector field
f = f + d1*[-x2;0] + d2*[0;-x1] + d1^2*[-x1+x2;0];
%%
% Once the vector field is written as above, then there is no difference
%%
% Get the options and system in the required form
[roaconstr,opt,sys] = GetRoaOpts(f, x);
opt.sim.NumConvTraj = 40;
%%
% call the wrapper which in term calls cellBetaCenter after trivial
% manipulation.
outputs = wrapper(sys,ini_cell,roaconstr,opt);
%%
% Beta upper and lower bounds:
[outputs.CellBetaInfo.infoOut.nominalinfo.BetaBnds(1) outputs.CellBetaInfo.infoOut.Beta]


%% Run B&B for the system in the previous case

%% 
% Modify opt to run B&B & and to change display options
opt.BB.runBB = 1;
opt.BB.max_iter = 3;
opt.display.BB = 0;
%% 
% call the wrapper which calls cellBetaCenter after trivial
% manipulation.
outputs = wrapper(sys,ini_cell,roaconstr,opt);

%%
% Upper and lower bound: 
[min(outputs.BBInfo.UpB_vec) min(outputs.BBInfo.Beta_vec(outputs.BBInfo.Active==1))]